"""controls for protocols interactions on the verity-application"""
__pdoc__ = {
    'CommittedAnswer': False,
    'IssuerSetup': False,
    'UpdateConfigs': False,
    'UpdateEndpoint': False,
    'WriteCredentialDefinition': False,
    'WriteSchema': False,
}
