# This is just a reimport for backwards compatibility
# This is deprecate and will be removed in future releases of SDK
from verity_sdk.protocols.v0_6.IssuerSetup import IssuerSetup  # pylint: disable=unused-import
